
// (c) 2003, Ivan Yanikov, dobrokot 'at' gmail.com 

unit mat;

interface
uses
  OpenGL;

type
  RealType=single;

  Vector=object
    x,y,z: RealType;
    procedure Let(newx, newy, newz: RealType);
    function Length:RealType;
    procedure Normalize;
  end;

  Matrix=record
    m: array [0..3, 0..3] of RealType;
  end;
  function mtAdd(const v1, v2:Vector):Vector;
  function mtSub(const v1, v2:Vector):Vector;
  function sMul(const v1, v2:Vector):RealType;
  function cMul(const v1, v2:Vector):Vector;
  function rMul(const v1:Vector; r:RealType):Vector;

  function mtMul(const v: Vector; const m:Matrix): Vector; overload;
  function mtMul(const m1, m2:Matrix): Matrix; overload;
  function mtRotX(angle: RealType):Matrix;
  function mtRotY(angle: RealType):Matrix;
  function mtRotZ(angle: RealType):Matrix;
  function mtTranslate(dx, dy, dz: RealType):Matrix;
  function mtIdentity: Matrix;

  procedure mtLoadToGL(const m:Matrix);
  procedure mtMulToGL(const m:Matrix);

implementation

function mtSub(const v1, v2:Vector):Vector;
begin
  Result.x:=v1.x-v2.x;
  Result.y:=v1.y-v2.y;
  Result.z:=v1.z-v2.z;
end;


function mtAdd(const v1, v2:Vector):Vector;
begin
  Result.x:=v1.x+v2.x;
  Result.y:=v1.y+v2.y;
  Result.z:=v1.z+v2.z;
end;


function sMul(const v1, v2:Vector):RealType;
begin
  sMul:=v1.x* v2.x+ v1.y* v2.y+ v1.z* v2.z;
end;


function cMul(const v1, v2:Vector):Vector;
begin
  Result.x:=v1.y*v2.z - v1.z*v2.y;
  Result.y:=v1.z*v2.x - v1.x*v2.z;
  Result.z:=v1.x*v2.y - v1.y*v2.x;
end;


function rMul(const v1:Vector; r:RealType):Vector;
begin
  Result.x:=v1.x*r;
  Result.y:=v1.y*r;
  Result.z:=v1.z*r;
end;


procedure Vector.Let(newx, newy, newz: RealType);
begin
  x:=newx;
  y:=newy;
  z:=newz;
end;


function Vector.Length:RealType;
begin
  Length:=sqrt(sqr(x)+sqr(y)+sqr(z));
end;


procedure Vector.Normalize;
var
  t:RealType;
begin
  t:=1/sqrt(sqr(x)+sqr(y)+sqr(z));
  x:=x*t;
  y:=y*t;
  z:=z*t;
end;


function mtRotX(angle: RealType):Matrix;
var
  i,j: integer;
begin
  for j:=0 to 3 do
    for i:=0 to 3 do
      Result.m[j,i]:=0;
  Result.m[3,3]:=1;
  Result.m[0,0]:=1;
  Result.m[1,1]:=cos(angle);
  Result.m[2,1]:=-sin(angle);
  Result.m[1,2]:=sin(angle);
  Result.m[2,2]:=cos(angle);
end;


function mtRotY(angle: RealType):Matrix;
var
  i,j: integer;
begin
  for j:=0 to 3 do
    for i:=0 to 3 do
      Result.m[j,i]:=0;
  Result.m[3,3]:=1;
  Result.m[1,1]:=1;
  Result.m[0,0]:=cos(angle);
  Result.m[2,0]:=sin(angle);
  Result.m[0,2]:=-sin(angle);
  Result.m[2,2]:=cos(angle);
end;


function mtRotZ(angle: RealType):Matrix;
var
  i,j: integer;
begin
  for j:=0 to 3 do
    for i:=0 to 3 do
      Result.m[j,i]:=0;
  Result.m[3,3]:=1;
  Result.m[2,2]:=1;
  Result.m[0,0]:=cos(angle);     
  Result.m[1,0]:=-sin(angle);
  Result.m[0,1]:=sin(angle);
  Result.m[1,1]:=cos(angle);
end;

function mtTranslate(dx, dy, dz: RealType):Matrix;
var
  i, j:integer;
begin
  for j:=0 to 3 do
    for i:=0 to 3 do
      Result.m[j,i]:=0;
  for j:=0 to 3 do
    Result.m[j,j]:=1;
  Result.m[3,0]:=dx;
  Result.m[3,1]:=dy;
  Result.m[3,2]:=dz;
end;


function mtMul(const v: Vector; const m:Matrix): Vector; overload;
begin
  Result.x:=m.m[0,0]*v.x+m.m[1,0]*v.y+m.m[2,0]*v.z+m.m[3,0];
  Result.y:=m.m[0,1]*v.x+m.m[1,1]*v.y+m.m[2,1]*v.z+m.m[3,1];
  Result.z:=m.m[0,2]*v.x+m.m[1,2]*v.y+m.m[2,2]*v.z+m.m[3,2];
end;


function mtMul(const m1, m2:Matrix): Matrix;
var
  i,j,k: integer;
  t: RealType;
begin
  for j:=0 to 3 do
    for i:=0 to 3 do begin
      t:=0;
      for k:=0 to 3 do
        t:=t+m1.m[k,i]*m2.m[j,k];
      Result.m[j,i]:=t;
    end;
end;


function mtIdentity: Matrix;
var
  i,j: integer;
begin
  for j:=0 to 3 do
    for i:=0 to 3 do
      Result.m[j,i]:=0;
  for j:=0 to 3 do
    Result.m[j,j]:=1;
end;


procedure mtLoadToGL(const m:Matrix);
begin
  glLoadMatrixf(@m.m[0,0]);
end;


procedure mtMulToGL(const m:Matrix);
begin
  glMultMatrixf(@m.m[0,0]);
end;


end.
